@@ -4,7 +4,6 @@ from __future__ import (absolute_import, division, print_function,
 import six
 
 import os, sys, warnings
-def fn_name(): return sys._getframe(1).f_code.co_name
 
 if six.PY3:
     warnings.warn(
@@ -29,24 +28,20 @@ _new_tooltip_api =  (gtk.pygtk_version[1] >= 12)
 
 import matplotlib
 from matplotlib._pylab_helpers import Gcf
-from matplotlib.backend_bases import RendererBase, GraphicsContextBase, \
-     FigureManagerBase, FigureCanvasBase, NavigationToolbar2, cursors, TimerBase
-from matplotlib.backend_bases import ShowBase
+from matplotlib.backend_bases import (
+    _Backend, FigureCanvasBase, FigureManagerBase, NavigationToolbar2,
+    TimerBase, cursors)
 
 from matplotlib.backends.backend_gdk import RendererGDK, FigureCanvasGDK
-from matplotlib.cbook import is_string_like, is_writable_file_like
+from matplotlib.cbook import is_writable_file_like, warn_deprecated
 from matplotlib.figure import Figure
 from matplotlib.widgets import SubplotTool
-from matplotlib.cbook import warn_deprecated
 
 from matplotlib import (
     cbook, colors as mcolors, lines, markers, rcParams, verbose)
 
 backend_version = "%d.%d.%d" % gtk.pygtk_version
 
-_debug = False
-#_debug = True
-
 # the true dots per inch on the screen; should be display dependent
 # see http://groups.google.com/groups?q=screen+dpi+x11&hl=en&lr=&ie=UTF-8&oe=UTF-8&safe=off&selm=7077.26e81ad5%40swift.cs.tcd.ie&rnum=5 for some info about screen dpi
 PIXELS_PER_INCH = 96
@@ -59,6 +54,7 @@ cursord = {
     cursors.HAND          : gdk.Cursor(gdk.HAND2),
     cursors.POINTER       : gdk.Cursor(gdk.LEFT_PTR),
     cursors.SELECT_REGION : gdk.Cursor(gdk.TCROSS),
+    cursors.WAIT          : gdk.Cursor(gdk.WATCH),
     }
 
 # ref gtk+/gtk/gtkwidget.h
@@ -67,53 +63,22 @@ def GTK_WIDGET_DRAWABLE(w):
     return flags & gtk.VISIBLE != 0 and flags & gtk.MAPPED != 0
 
 
-def draw_if_interactive():
-    """
-    Is called after every pylab drawing command
-    """
-    if matplotlib.is_interactive():
-        figManager =  Gcf.get_active()
-        if figManager is not None:
-            figManager.canvas.draw_idle()
-
-
-class Show(ShowBase):
-    def mainloop(self):
-        if gtk.main_level() == 0:
-            gtk.main()
-
-show = Show()
-
-def new_figure_manager(num, *args, **kwargs):
-    """
-    Create a new figure manager instance
-    """
-    FigureClass = kwargs.pop('FigureClass', Figure)
-    thisFig = FigureClass(*args, **kwargs)
-    return new_figure_manager_given_figure(num, thisFig)
-
-
-def new_figure_manager_given_figure(num, figure):
-    """
-    Create a new figure manager instance for the given figure.
-    """
-    canvas = FigureCanvasGTK(figure)
-    manager = FigureManagerGTK(canvas, num)
-    return manager
-
-
 class TimerGTK(TimerBase):
     '''
-    Subclass of :class:`backend_bases.TimerBase` that uses GTK for timer events.
+    Subclass of :class:`backend_bases.TimerBase` using GTK for timer events.
+
+    Attributes
+    ----------
+    interval : int
+        The time between timer events in milliseconds. Default is 1000 ms.
+    single_shot : bool
+        Boolean flag indicating whether this timer should operate as single
+        shot (run once and then stop). Defaults to False.
+    callbacks : list
+        Stores list of (func, args) tuples that will be called upon timer
+        events. This list can be manipulated directly, or the functions
+        `add_callback` and `remove_callback` can be used.
 
-    Attributes:
-    * interval: The time between timer events in milliseconds. Default
-        is 1000 ms.
-    * single_shot: Boolean flag indicating whether this timer should
-        operate as single shot (run once and then stop). Defaults to False.
-    * callbacks: Stores list of (func, args) tuples that will be called
-        upon timer events. This list can be manipulated directly, or the
-        functions add_callback and remove_callback can be used.
     '''
     def _timer_start(self):
         # Need to stop it, otherwise we potentially leak a timer id that will
@@ -221,7 +186,6 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
                             "See Matplotlib usage FAQ for"
                             " more info on backends.",
                             alternative="GTKAgg")
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         FigureCanvasBase.__init__(self, figure)
         gtk.DrawingArea.__init__(self)
 
@@ -257,7 +221,6 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
             gobject.source_remove(self._idle_draw_id)
 
     def scroll_event(self, widget, event):
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         x = event.x
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - event.y
@@ -269,7 +232,6 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         return False  # finish event propagation?
 
     def button_press_event(self, widget, event):
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         x = event.x
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - event.y
@@ -293,7 +255,6 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         return False  # finish event propagation?
 
     def button_release_event(self, widget, event):
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         x = event.x
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - event.y
@@ -301,21 +262,16 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         return False  # finish event propagation?
 
     def key_press_event(self, widget, event):
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         key = self._get_key(event)
-        if _debug: print("hit", key)
         FigureCanvasBase.key_press_event(self, key, guiEvent=event)
         return True  # stop event propagation
 
     def key_release_event(self, widget, event):
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         key = self._get_key(event)
-        if _debug: print("release", key)
         FigureCanvasBase.key_release_event(self, key, guiEvent=event)
         return True  # stop event propagation
 
     def motion_notify_event(self, widget, event):
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         if event.is_hint:
             x, y, state = event.window.get_pointer()
         else:
@@ -351,7 +307,6 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         return key
 
     def configure_event(self, widget, event):
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         if widget.window is None:
             return
         w, h = event.width, event.height
@@ -404,8 +359,6 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         Make sure _._pixmap is at least width, height,
         create new pixmap if necessary
         """
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
-
         create_pixmap = False
         if width > self._pixmap_width:
             # increase the pixmap in 10%+ (rather than 1 pixel) steps
@@ -434,18 +387,20 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
     def expose_event(self, widget, event):
         """Expose_event for all GTK backends. Should not be overridden.
         """
-        if _debug: print('FigureCanvasGTK.%s' % fn_name())
-
+        toolbar = self.toolbar
+        # if toolbar:
+        #     toolbar.set_cursor(cursors.WAIT)
         if GTK_WIDGET_DRAWABLE(self):
             if self._need_redraw:
                 x, y, w, h = self.allocation
                 self._pixmap_prepare (w, h)
                 self._render_figure(self._pixmap, w, h)
                 self._need_redraw = False
-
             x, y, w, h = event.area
             self.window.draw_drawable (self.style.fg_gc[self.state],
                                        self._pixmap, x, y, x, y, w, h)
+        # if toolbar:
+        #     toolbar.set_cursor(toolbar._lastCursor)
         return False  # finish event propagation?
 
     filetypes = FigureCanvasBase.filetypes.copy()
@@ -479,14 +434,12 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
 
         # set the default quality, if we are writing a JPEG.
         # http://www.pygtk.org/docs/pygtk/class-gdkpixbuf.html#method-gdkpixbuf--save
-        options = cbook.restrict_dict(kwargs, ['quality'])
-        if format in ['jpg','jpeg']:
-            if 'quality' not in options:
-                options['quality'] = rcParams['savefig.jpeg_quality']
-
+        options = {k: kwargs[k] for k in ['quality'] if k in kwargs}
+        if format in ['jpg', 'jpeg']:
+            options.setdefault('quality', rcParams['savefig.jpeg_quality'])
             options['quality'] = str(options['quality'])
 
-        if is_string_like(filename):
+        if isinstance(filename, six.string_types):
             try:
                 pixbuf.save(filename, format, options=options)
             except gobject.GError as exc:
@@ -510,13 +463,13 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         This is useful for getting periodic events through the backend's native
         event loop. Implemented only for backends with GUIs.
 
-        optional arguments:
-
-        *interval*
-          Timer interval in milliseconds
-        *callbacks*
-          Sequence of (func, args, kwargs) where func(*args, **kwargs) will
-          be executed by the timer every *interval*.
+        Other Parameters
+        ----------------
+        interval : scalar
+            Timer interval in milliseconds
+        callbacks : list
+            Sequence of (func, args, kwargs) where ``func(*args, **kwargs)``
+            will be executed by the timer every *interval*.
         """
         return TimerGTK(*args, **kwargs)
 
@@ -527,31 +480,30 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         gtk.gdk.flush()
         gtk.gdk.threads_leave()
 
-    def start_event_loop(self,timeout):
-        FigureCanvasBase.start_event_loop_default(self,timeout)
-    start_event_loop.__doc__=FigureCanvasBase.start_event_loop_default.__doc__
-
-    def stop_event_loop(self):
-        FigureCanvasBase.stop_event_loop_default(self)
-    stop_event_loop.__doc__=FigureCanvasBase.stop_event_loop_default.__doc__
 
 class FigureManagerGTK(FigureManagerBase):
     """
-    Public attributes
+    Attributes
+    ----------
+    canvas : `FigureCanvas`
+        The FigureCanvas instance
+    num : int or str
+        The Figure number
+    toolbar : gtk.Toolbar
+        The gtk.Toolbar  (gtk only)
+    vbox : gtk.VBox
+        The gtk.VBox containing the canvas and toolbar (gtk only)
+    window : gtk.Window
+        The gtk.Window   (gtk only)
 
-    canvas      : The FigureCanvas instance
-    num         : The Figure number
-    toolbar     : The gtk.Toolbar  (gtk only)
-    vbox        : The gtk.VBox containing the canvas and toolbar (gtk only)
-    window      : The gtk.Window   (gtk only)
     """
     def __init__(self, canvas, num):
-        if _debug: print('FigureManagerGTK.%s' % fn_name())
         FigureManagerBase.__init__(self, canvas, num)
 
         self.window = gtk.Window()
+        self.window.set_wmclass("matplotlib", "Matplotlib")
         self.set_window_title("Figure %d" % num)
-        if (window_icon):
+        if window_icon:
             try:
                 self.window.set_icon_from_file(window_icon)
             except:
@@ -599,7 +551,6 @@ class FigureManagerGTK(FigureManagerBase):
         self.canvas.grab_focus()
 
     def destroy(self, *args):
-        if _debug: print('FigureManagerGTK.%s' % fn_name())
         if hasattr(self, 'toolbar') and self.toolbar is not None:
             self.toolbar.destroy()
         if hasattr(self, 'vbox'):
@@ -618,6 +569,9 @@ class FigureManagerGTK(FigureManagerBase):
     def show(self):
         # show the figure window
         self.window.show()
+        # raise the window above others and relase the "above lock"
+        self.window.set_keep_above(True)
+        self.window.set_keep_above(False)
 
     def full_screen_toggle(self):
         self._full_screen_flag = not self._full_screen_flag
@@ -662,15 +616,12 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
 
     def set_cursor(self, cursor):
         self.canvas.window.set_cursor(cursord[cursor])
+        gtk.main_iteration()
 
     def release(self, event):
         try: del self._pixmapBack
         except AttributeError: pass
 
-    def dynamic_update(self):
-        # legacy method; new method is canvas.draw_idle
-        self.canvas.draw_idle()
-
     def draw_rubberband(self, event, x0, y0, x1, y1):
         'adapted from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/189744'
         drawable = self.canvas.window
@@ -748,7 +699,7 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         fc = FileChooserDialog(
             title='Save the figure',
             parent=self.win,
-            path=os.path.expanduser(rcParams.get('savefig.directory', '')),
+            path=os.path.expanduser(rcParams['savefig.directory']),
             filetypes=self.canvas.get_supported_filetypes(),
             default_filetype=self.canvas.get_default_filetype())
         fc.set_current_name(self.canvas.get_default_filename())
@@ -759,15 +710,13 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         fname, format = chooser.get_filename_from_user()
         chooser.destroy()
         if fname:
-            startpath = os.path.expanduser(rcParams.get('savefig.directory', ''))
-            if startpath == '':
-                # explicitly missing key or empty str signals to use cwd
-                rcParams['savefig.directory'] = startpath
-            else:
-                # save dir for next time
-                rcParams['savefig.directory'] = os.path.dirname(six.text_type(fname))
+            startpath = os.path.expanduser(rcParams['savefig.directory'])
+            # Save dir for next time, unless empty str (i.e., use cwd).
+            if startpath != "":
+                rcParams['savefig.directory'] = (
+                    os.path.dirname(six.text_type(fname)))
             try:
-                self.canvas.print_figure(fname, format=format)
+                self.canvas.figure.savefig(fname, format=format)
             except Exception as e:
                 error_msg_gtk(str(e), parent=self)
 
@@ -777,13 +726,13 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         toolfig.subplots_adjust(top=0.9)
         tool =  SubplotTool(self.canvas.figure, toolfig)
 
-        w = int (toolfig.bbox.width)
-        h = int (toolfig.bbox.height)
-
+        w = int(toolfig.bbox.width)
+        h = int(toolfig.bbox.height)
 
         window = gtk.Window()
-        if (window_icon):
-            try: window.set_icon_from_file(window_icon)
+        if window_icon:
+            try:
+                window.set_icon_from_file(window_icon)
             except:
                 # we presumably already logged a message on the
                 # failure of the main plot, don't keep reporting
@@ -816,33 +765,32 @@ class FileChooserDialog(gtk.FileChooserDialog):
                   filetypes = [],
                   default_filetype = None
                   ):
-        super(FileChooserDialog, self).__init__ (title, parent, action,
-                                                 buttons)
+        super(FileChooserDialog, self).__init__(title, parent, action, buttons)
         super(FileChooserDialog, self).set_do_overwrite_confirmation(True)
-        self.set_default_response (gtk.RESPONSE_OK)
+        self.set_default_response(gtk.RESPONSE_OK)
 
-        if not path: path = os.getcwd() + os.sep
+        if not path:
+            path = os.getcwd() + os.sep
 
         # create an extra widget to list supported image formats
         self.set_current_folder (path)
         self.set_current_name ('image.' + default_filetype)
 
-        hbox = gtk.HBox (spacing=10)
-        hbox.pack_start (gtk.Label ("File Format:"), expand=False)
+        hbox = gtk.HBox(spacing=10)
+        hbox.pack_start(gtk.Label ("File Format:"), expand=False)
 
         liststore = gtk.ListStore(gobject.TYPE_STRING)
         cbox = gtk.ComboBox(liststore)
         cell = gtk.CellRendererText()
         cbox.pack_start(cell, True)
         cbox.add_attribute(cell, 'text', 0)
-        hbox.pack_start (cbox)
+        hbox.pack_start(cbox)
 
         self.filetypes = filetypes
-        self.sorted_filetypes = list(six.iteritems(filetypes))
-        self.sorted_filetypes.sort()
+        self.sorted_filetypes = sorted(six.iteritems(filetypes))
         default = 0
         for i, (ext, name) in enumerate(self.sorted_filetypes):
-            cbox.append_text ("%s (*.%s)" % (name, ext))
+            cbox.append_text("%s (*.%s)" % (name, ext))
             if ext == default_filetype:
                 default = i
         cbox.set_active(default)
@@ -861,8 +809,8 @@ class FileChooserDialog(gtk.FileChooserDialog):
             elif ext == '':
                 filename = filename.rstrip('.') + '.' + new_ext
 
-            self.set_current_name (filename)
-        cbox.connect ("changed", cb_cbox_changed)
+            self.set_current_name(filename)
+        cbox.connect("changed", cb_cbox_changed)
 
         hbox.show_all()
         self.set_extra_widget(hbox)
@@ -877,6 +825,7 @@ class FileChooserDialog(gtk.FileChooserDialog):
 
         return filename, self.ext
 
+
 class DialogLineprops(object):
     """
     A GUI dialog for controlling lineprops
@@ -892,12 +841,11 @@ class DialogLineprops(object):
         )
 
     linestyles = [ls for ls in lines.Line2D.lineStyles if ls.strip()]
-    linestyled = dict([ (s,i) for i,s in enumerate(linestyles)])
+    linestyled = {s: i for i, s in enumerate(linestyles)}
 
-
-    markers =  [m for m in markers.MarkerStyle.markers if cbook.is_string_like(m)]
-
-    markerd = dict([(s,i) for i,s in enumerate(markers)])
+    markers =  [m for m in markers.MarkerStyle.markers
+                if isinstance(m, six.string_types)]
+    markerd = {s: i for i, s in enumerate(markers)}
 
     def __init__(self, lines):
         import gtk.glade
@@ -905,12 +853,14 @@ class DialogLineprops(object):
         datadir = matplotlib.get_data_path()
         gladefile = os.path.join(datadir, 'lineprops.glade')
         if not os.path.exists(gladefile):
-            raise IOError('Could not find gladefile lineprops.glade in %s'%datadir)
+            raise IOError(
+                'Could not find gladefile lineprops.glade in %s' % datadir)
 
         self._inited = False
         self._updateson = True # suppress updates when setting widgets manually
         self.wtree = gtk.glade.XML(gladefile, 'dialog_lineprops')
-        self.wtree.signal_autoconnect(dict([(s, getattr(self, s)) for s in self.signals]))
+        self.wtree.signal_autoconnect(
+            {s: getattr(self, s) for s in self.signals})
 
         self.dlg = self.wtree.get_widget('dialog_lineprops')
 
@@ -934,7 +884,6 @@ class DialogLineprops(object):
         self._lastcnt = 0
         self._inited = True
 
-
     def show(self):
         'populate the combo box'
         self._updateson = False
@@ -958,7 +907,6 @@ class DialogLineprops(object):
         line = self.lines[ind]
         return line
 
-
     def get_active_linestyle(self):
         'get the active lineinestyle'
         ind = self.cbox_linestyles.get_active()
@@ -992,8 +940,6 @@ class DialogLineprops(object):
 
         line.figure.canvas.draw()
 
-
-
     def on_combobox_lineprops_changed(self, item):
         'update the widgets from the active line'
         if not self._inited: return
@@ -1043,7 +989,6 @@ class DialogLineprops(object):
 # Unfortunately, the SVG renderer (rsvg) leaks memory under earlier
 # versions of pygtk, so we have to use a PNG file instead.
 try:
-
     if gtk.pygtk_version < (2, 8, 0) or sys.platform == 'win32':
         icon_filename = 'matplotlib.png'
     else:
@@ -1059,7 +1004,7 @@ def error_msg_gtk(msg, parent=None):
         if parent.flags() & gtk.TOPLEVEL == 0:
             parent = None
 
-    if not is_string_like(msg):
+    if not isinstance(msg, six.string_types):
         msg = ','.join(map(str,msg))
 
     dialog = gtk.MessageDialog(
@@ -1071,5 +1016,16 @@ def error_msg_gtk(msg, parent=None):
     dialog.destroy()
 
 
-FigureCanvas = FigureCanvasGTK
-FigureManager = FigureManagerGTK
+@_Backend.export
+class _BackendGTK(_Backend):
+    FigureCanvas = FigureCanvasGTK
+    FigureManager = FigureManagerGTK
+
+    @staticmethod
+    def trigger_manager_draw(manager):
+        manager.canvas.draw_idle()
+
+    @staticmethod
+    def mainloop():
+        if gtk.main_level() == 0:
+            gtk.main()
